@@ -26,11 +26,12 @@
 #define MIXGTK_CMD_DISPATCHER_H
 
 #include <mixlib/mix_vm.h>
+#include <mixlib/mix_vm_command.h>
 #include "mixgtk_widgets.h"
 
 /* initialise the command dispatcher */
 extern gboolean
-mixgtk_cmd_dispatcher_init (void);
+mixgtk_cmd_dispatcher_init (mixgtk_dialog_id_t top);
 
 /* dispatch an externally provided command */
 extern void
@@ -48,6 +49,10 @@ mixgtk_cmd_dispatcher_get_vm (void);
 extern const gchar *
 mixgtk_cmd_dispatcher_get_src_path (void);
 
+/* get the mix cmd dispatcher */
+extern mix_vm_cmd_dispatcher_t *
+mixgtk_cmd_dispatcher_get_mix_dispatcher (void);
+
 
 #endif /* MIXGTK_CMD_DISPATCHER_H */
 
