@@ -30,6 +30,10 @@
 /* callback function type */
 typedef void (*input_callback_t)(mix_word_t result, gpointer data);
 
+/* init */
+extern void
+mixgtk_input_init (void);
+
 /* get a word */
 extern void
 mixgtk_input_word (const gchar *message, mix_word_t def,
