@@ -1,9 +1,9 @@
 /* -*-c-*- -------------- mixgtk_mixvm.c :
  * Implementation of the functions declared in mixgtk_mixvm.h
  * ------------------------------------------------------------------
- *  Last change: Time-stamp: "2001-05-19 16:39:16 jao"
+ * $Id: mixgtk_mixvm.c,v 1.10 2002/04/10 23:39:40 jao Exp $
  * ------------------------------------------------------------------
- * Copyright (C) 2001 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2002 Free Software Foundation, Inc.
  *  
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -91,7 +91,7 @@ update_register_ (mixgtk_widget_id_t reg)
       g_assert_not_reached ();
     }
   gtk_entry_set_text (reg_entries_[reg - MIXGTK_WIDGET_rA], BUFFER);
-  snprintf (BUFFER, 20, "%s%ld", mix_word_is_negative (tipval)? "-" : "",
+  g_snprintf (BUFFER, 20, "%s%ld", mix_word_is_negative (tipval)? "-" : "",
 	    mix_word_magnitude (tipval));
   gtk_tooltips_set_tip (tips_,
 			GTK_WIDGET (reg_entries_[reg - MIXGTK_WIDGET_rA]),
@@ -160,7 +160,7 @@ mixgtk_mixvm_init (mix_vm_t *vm, mixgtk_dialog_id_t top)
   for (k = 0; k< MIX_VM_CELL_NO; ++k)
     {
       gchar buffer[20];
-      sprintf (buffer, "%04d", k);
+      g_snprintf (buffer, 20, "%04d", k);
       text[0] = buffer;
       gtk_clist_append (memory_, text);
     }
@@ -254,7 +254,7 @@ mixgtk_mixvm_update_loc (void)
   g_assert (loc_entry_);
   
   locval = mix_vm_get_prog_count (vm_);
-  snprintf (BUFFER, SIZE, "%04d", mix_short_magnitude (locval));
+  g_snprintf (BUFFER, SIZE, "%04d", mix_short_magnitude (locval));
   gtk_entry_set_text (loc_entry_, BUFFER);
 }
 
@@ -268,11 +268,11 @@ mixgtk_mixvm_update_times (void)
   
   g_assert (uptime_ && progtime_ && laptime_);
   mixgtk_cmd_dispatcher_get_times (&uptime, &progtime, &laptime);
-  snprintf (BUFFER, SIZE, "%d", uptime);
+  g_snprintf (BUFFER, SIZE, "%d", uptime);
   gtk_label_set_text (uptime_, BUFFER);
-  snprintf (BUFFER, SIZE, "%d", progtime);
+  g_snprintf (BUFFER, SIZE, "%d", progtime);
   gtk_label_set_text (progtime_, BUFFER);
-  snprintf (BUFFER, SIZE, "%d", laptime);
+  g_snprintf (BUFFER, SIZE, "%d", laptime);
   gtk_label_set_text (laptime_, BUFFER);
 }
 
@@ -361,18 +361,19 @@ on_register_click (GtkWidget *w, GdkEvent *e, gpointer data)
   gint k;
   
   if (w == GTK_WIDGET (reg_entries_[0]))
-    mixgtk_input_word ("Register A", mix_vm_get_rA (vm_),
+    mixgtk_input_word (_("Register A"), mix_vm_get_rA (vm_),
 		       register_AX_, GINT_TO_POINTER (0));
   else if (w == GTK_WIDGET (reg_entries_[1]))
-    mixgtk_input_word ("Register X", mix_vm_get_rX (vm_),
+    mixgtk_input_word (_("Register X"), mix_vm_get_rX (vm_),
 		       register_AX_, GINT_TO_POINTER(1));
   else if (w == GTK_WIDGET (reg_entries_[2]))
-    mixgtk_input_short ("Register J", mix_vm_get_rJ (vm_), register_j_, NULL);
+    mixgtk_input_short (_("Register J"), mix_vm_get_rJ (vm_),
+			register_j_, NULL);
   else for (k = 1; k < 7; ++k)
     {
       if (w == GTK_WIDGET (reg_entries_[2 + k]))
 	{
-	  snprintf (BUFFER, SIZE, "Register I%d", k);
+	  g_snprintf (BUFFER, SIZE, _("Register I%d"), k);
 	  mixgtk_input_short (BUFFER, mix_vm_get_rI (vm_, k),
 			      register_i_, GINT_TO_POINTER (k));
 	  break;
@@ -395,7 +396,7 @@ on_memory_cells_select_row (GtkCList *w, gint row, gint col,
 {
   enum {SIZE = 30};
   static gchar BUFFER[SIZE];
-  snprintf (BUFFER, SIZE, "Memory cell no. %d", row);
+  g_snprintf (BUFFER, SIZE, _("Memory cell no. %d"), row);
   mixgtk_input_word (BUFFER,
 		     mix_vm_get_addr_contents (vm_, mix_short_new (row)),
 		     mem_cell_, GINT_TO_POINTER (row));
