@@ -1,7 +1,7 @@
 /* -*-c-*- ---------------- mix.h :
  * Initialisation of the mix library
  * ------------------------------------------------------------------
- *  $Id: mix.h,v 1.5 2001/09/29 01:54:30 jao Exp $
+ *  $Id: mix.h,v 1.6 2002/12/08 00:35:54 jao Exp $
  * ------------------------------------------------------------------
  * Copyright (C) 2000, 2001 Free Software Foundation, Inc.
  *  
@@ -37,12 +37,15 @@
 #	include <locale.h>
 #endif
 
+/*
 #ifdef HAVE_GETTEXT
 #	include <libintl.h>
 #	define gettext_noop(String)	(String)
 #else
 #	include <intl/libgettext.h>
 #endif 
+*/
+#include "gettext.h"
 
 #define _(String) gettext (String)
 #define N_(String) gettext_noop (String)
