@@ -2,7 +2,7 @@
  * Implementation of the functions declared in mix_parser.h and
  * xmix_parser.h
  * ------------------------------------------------------------------
- * Copyright (C) 2000, 2001, 2003 Free Software Foundation, Inc.
+ * Copyright (C) 2000, 2001, 2003, 2004 Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -99,7 +99,7 @@ mix_parser_src_file_extension (const mix_parser_t *parser)
 static gint
 compare_shorts_ (gconstpointer s1, gconstpointer s2)
 {
-  mix_short_t a = (mix_short_t)GPOINTER_TO_UINT(s1);
+  mix_short_t a = (mix_short_t)GPOINTER_TO_UINT (s1);
   mix_short_t b = (mix_short_t)GPOINTER_TO_UINT (s2);
   if ( mix_short_sign (a) == mix_short_sign (b) )
     return mix_short_magnitude (a) - mix_short_magnitude (b);
@@ -136,8 +136,12 @@ mix_parser_new (const gchar *in_file)
       g_warning (_("No system resources"));
       return NULL;
     }
+  result->con_list = NULL;
+  result->alf_list = NULL;
   result->in_file = f;
   result->loc_count = MIX_SHORT_ZERO;
+  result->start = MIX_SHORT_ZERO;
+  result->end = MIX_SHORT_ZERO;
   result->status = MIX_PERR_NOCOMP;
   result->err_line = 0;
   result->err_count = 0;
@@ -172,39 +176,32 @@ mix_parser_delete (mix_parser_t *parser)
   mix_symbol_table_delete (parser->symbol_table);
   mix_symbol_table_delete (parser->ls_table);
   g_hash_table_destroy (parser->future_refs);
+  g_slist_free (parser->con_list);
+  g_slist_free (parser->alf_list);
   mix_file_delete (parser->in_file);
   g_free (parser);
 }
 
 /* Compile a mix source file */
-static gboolean
-undef_warning_ (gpointer symbol, gpointer value, gpointer data)
-{
-  mix_parser_log_error ((mix_parser_t *)data, MIX_PERR_UNDEF_SYM, 1,
-			(const gchar *)symbol, TRUE);
-  /* move the symbol to the symbol table */
-  mix_symbol_table_insert_static (((mix_parser_t *)data)->symbol_table,
-				  symbol, MIX_WORD_ZERO);
-  return TRUE;
-}
-
-static void
+static guint
 update_future_refs_value_ (mix_parser_t *parser, const gchar *name,
-			   mix_short_t value)
+			   mix_short_t value, gboolean remove)
 {
   GSList *list = NULL;
+  gpointer *plist = (gpointer *)(&list);
   gpointer key;
+  guint line = 0;
+
 
   g_assert (parser != NULL && name != NULL);
-  if ( g_hash_table_lookup_extended (parser->future_refs, name, &key,
-				     (gpointer *)&list) )
+  if ( g_hash_table_lookup_extended (parser->future_refs, name, &key, plist) )
     {
       GSList *tmp = list;
       ins_node_ *node;
       while ( tmp != NULL )
 	{
 	  node =
-	    (ins_node_ *)g_tree_lookup (parser->ins_table,tmp->data);
+	    (ins_node_ *)g_tree_lookup (parser->ins_table, tmp->data);
 	  g_assert (node);
 	  if (mix_get_ins_address (node->ins) == 1) {
 	    value = mix_short_negative (value);
@@ -214,16 +211,21 @@ update_future_refs_value_ (mix_parser_t *parser, const gchar *name,
 	  }
 	  mix_word_add_address (node->ins, value);
 	  g_tree_insert (parser->ins_table, tmp->data, (gpointer)node);
+          /* record the first line the ref appears */
+          if (line == 0 || line > node->lineno) line = node->lineno;
 	  tmp = g_slist_next (tmp);
 	}
-      g_hash_table_remove (parser->future_refs, name);
-      g_free (key);
+      if (remove) {
+        g_hash_table_remove (parser->future_refs, name);
+        g_free (key);
+      }
       g_slist_free (list);
     }
+  return line;
 }
 
-#define update_future_refs_(parser,name) \
-  update_future_refs_value_(parser, name, (parser)->loc_count);
+#define update_future_refs_(parser,name,rem) \
+  update_future_refs_value_(parser, name, (parser)->loc_count, rem)
 
 static void
 update_ls_ (gpointer symbol, gpointer value, gpointer parser)
@@ -233,11 +235,29 @@ update_ls_ (gpointer symbol, gpointer value, gpointer parser)
   mix_parser_t *par = (mix_parser_t *) parser;
 
   mix_word_to_ins_uncheck (w, ins);
-  mix_parser_add_ins (par, &ins, 0);
-  update_future_refs_ (par, (const gchar *)symbol);
+  mix_parser_add_ins (par, &ins,
+                      update_future_refs_ (par, (const gchar *)symbol, TRUE));
   par->loc_count++;
 }
 
+static gboolean
+undef_warning_ (gpointer symbol, gpointer value, gpointer data)
+{
+  mix_parser_t *parser = (mix_parser_t *)data;
+  const gchar *name = (const gchar *)symbol;
+  mix_ins_t ins;
+
+  mix_parser_log_error (parser, MIX_PERR_UNDEF_SYM, 0, name, TRUE);
+
+  mix_word_to_ins_uncheck (MIX_WORD_ZERO, ins);
+  mix_parser_add_ins (parser, &ins, update_future_refs_ (parser, name, FALSE));
+  mix_symbol_table_insert (parser->symbol_table, name,
+                           mix_short_to_word_fast (parser->loc_count));
+
+  parser->loc_count++;
+  return TRUE;
+}
+
 mix_parser_err_t
 mix_parser_compile (mix_parser_t *parser)
 {
@@ -248,12 +268,16 @@ mix_parser_compile (mix_parser_t *parser)
   g_return_val_if_fail (parser->ins_table != NULL, MIX_PERR_INTERNAL);
 
   parser->status = mix_flex_scan (parser);
+
   if ( parser->status == MIX_PERR_OK )
     {
-      mix_symbol_table_foreach (parser->ls_table, update_ls_,
-				(gpointer)parser);
-      g_hash_table_foreach_remove (parser->future_refs,
-				   undef_warning_, (gpointer)parser);
+      parser->loc_count = parser->end;
+      mix_symbol_table_foreach (parser->ls_table, update_ls_, (gpointer)parser);
+      if ( g_hash_table_size (parser->future_refs) > 0)
+        {
+          g_hash_table_foreach_remove (parser->future_refs,
+                                       undef_warning_, (gpointer)parser);
+        }
     }
 
   return parser->status;
@@ -317,53 +341,90 @@ mix_parser_write_code (mix_parser_t *parser, const gchar *code_file,
 }
 
 /* Produce a listing file summarising the compilation */
+typedef struct
+{
+  FILE *file;
+  mix_parser_t *parser;
+} listing_context_t;
+
 static gint
 write_listing_ (gpointer address, gpointer ins, gpointer context)
 {
   guint k;
-  FILE *file = (FILE *)context;
+  FILE *file = ((listing_context_t *)context)->file;
+  mix_parser_t *parser = ((listing_context_t *)context)->parser;
+  guint end = parser->end;
   ins_node_ *ins_node = (ins_node_ *)ins;
   mix_ins_t instruct;
-  gchar *instext = NULL;
 
-  mix_ins_id_t id = mix_word_to_ins (ins_node->ins, &instruct);
-  if (id != mix_INVALID_INS)
-    instext = mix_ins_to_string (&instruct);
-  fprintf (file, "%d\t%s ", GPOINTER_TO_INT (address),
+  fprintf (file, "%03d     %05d   %s ",
+           ins_node->lineno,
+           GPOINTER_TO_INT (address),
 	   mix_word_is_negative (ins_node->ins)? "-":"+");
   for ( k = 1; k < 6; ++k )
     fprintf (file, "%02d ", mix_word_get_byte (ins_node->ins, k));
-  fprintf (file, _("\t%s\t(line %d)\n"), instext? instext:"--DATA--",
-	   ins_node->lineno);
-  if (instext) g_free (instext);
+
+  if (g_slist_find (parser->con_list, GUINT_TO_POINTER (ins_node->lineno))
+      || GPOINTER_TO_UINT (address) >=  end)
+    fprintf (file, "\tCON\t%04d\n", (int)(ins_node->ins));
+  else if (g_slist_find (parser->alf_list, GUINT_TO_POINTER (ins_node->lineno)))
+    {
+      size_t i;
+      fprintf (file, "\tALF\t\"");
+      for (i = 1; i < 6; ++i)
+        fprintf (file, "%c",
+                 mix_char_to_ascii (mix_byte_to_char
+                                    (mix_word_get_byte (ins_node->ins, i))));
+      fprintf (file, "\"\n");
+    }
+  else if (GPOINTER_TO_UINT (address) <  end)
+    {
+      gchar *instext = NULL;
+      mix_ins_id_t id = mix_word_to_ins (ins_node->ins, &instruct);
+      if (id != mix_INVALID_INS)
+        instext = mix_ins_to_string (&instruct);
+      fprintf (file, _("\t%s\n"), instext? instext : _("UNKNOWN"));
+      if (instext) g_free (instext);
+    }
+  else
+    g_assert_not_reached ();
+
   return FALSE;
 }
 
 mix_parser_err_t
 mix_parser_write_listing (mix_parser_t *parser, const gchar *list_file)
 {
-  FILE *file;
   mix_file_t *mfile;
   const gchar *name;
+  listing_context_t context;
+  static const char *sep =
+    "-----------------------------------------------------------------\n";
 
   g_return_val_if_fail (parser != NULL, MIX_PERR_INTERNAL);
   if (parser->status != MIX_PERR_OK ) return parser->status;
   name = (list_file) ? list_file : mix_file_base_name (parser->in_file);
   mfile =  mix_file_new_with_def_ext (name, mix_io_WRITE, MIX_LIST_DEFEXT);
   if ( mfile == NULL ) return MIX_PERR_NOOUT;
-  file = mix_file_to_FILE (mfile);
-  fprintf (file, _("*** %s%s: compilation summary ***\n\n"),
+  context.file = mix_file_to_FILE (mfile);
+  context.parser = parser;
+  fprintf (context.file, _("*** %s%s: compilation summary ***\n\n"),
 	   mix_file_base_name (parser->in_file),
 	   mix_file_extension (parser->in_file));
-  fputs ( _("*** Address / Compiled word / Symbolic rep / Source file line\n"),
-	  file);
+  fputs (sep, context.file);
+  fputs ( _("Src     Address  Compiled word           Symbolic rep\n"),
+	  context.file);
+  fputs (sep, context.file);
   g_tree_traverse (parser->ins_table, write_listing_,
-		   G_IN_ORDER, (gpointer)file);
-  fprintf (file, _("\n*** Start address: %d\n"),
-	   mix_short_magnitude (parser->start));
-  fprintf (file, _("\n*** Symbol table\n"));
-  mix_symbol_table_print (parser->symbol_table, MIX_SYM_ROWS, file, TRUE);
-  fprintf (file, _("\n*** End of summary ***\n"));
+                   G_IN_ORDER, (gpointer)(&context));
+  fputs (sep, context.file);
+  fprintf (context.file, _("\n*** Start address:\t%d\n*** End address:\t%d\n"),
+	   mix_short_magnitude (parser->start),
+           mix_short_magnitude (parser->end));
+  fprintf (context.file, _("\n*** Symbol table\n"));
+  mix_symbol_table_print (parser->symbol_table, MIX_SYM_ROWS,
+                          context.file, TRUE);
+  fprintf (context.file, _("\n*** End of summary ***\n"));
   mix_file_delete (mfile);
   return parser->status;
 }
@@ -417,7 +478,7 @@ mix_parser_define_symbol_value (mix_parser_t *parser, const gchar *name,
     {
     case MIX_SYM_OK:
       if ( parser->status == MIX_PERR_NOCOMP )
-	update_future_refs_value_ (parser, name, value);
+	update_future_refs_value_ (parser, name, value, TRUE);
       return MIX_PERR_OK;
     case MIX_SYM_LONG: return MIX_PERR_LONG_SYMBOL;
     case MIX_SYM_DUP: return MIX_PERR_DUP_SYMBOL;
@@ -463,7 +524,7 @@ mix_parser_manage_local_symbol (mix_parser_t *parser, const gchar *name,
       ref[0] = name[0];
       ref[1] = 'F';
       if ( parser->status == MIX_PERR_NOCOMP )
-	update_future_refs_value_ (parser, ref, value);
+	update_future_refs_value_ (parser, ref, value, TRUE);
       ref[1] = 'B';
       mix_symbol_table_insert (parser->symbol_table, ref,
 			       mix_short_to_word_fast (value));
@@ -484,18 +545,9 @@ mix_parser_define_ls (mix_parser_t *parser, mix_word_t value)
 }
 
 /* Compilation */
-void
-mix_parser_add_ins (mix_parser_t *parser, const mix_ins_t *new_ins,
-		    guint lineno)
+static void
+add_raw_ (mix_parser_t *parser, mix_word_t word, guint lineno)
 {
-  g_assert (parser != NULL && new_ins != NULL);
-  mix_parser_add_raw (parser, mix_ins_to_word_uncheck (*new_ins), lineno);
-}
-
-void
-mix_parser_add_raw (mix_parser_t *parser, mix_word_t word, guint lineno)
-{
-  g_assert (parser != NULL);
   if ( parser->status == MIX_PERR_NOCOMP || parser->status == MIX_PERR_OK )
     {
       ins_node_ *node = g_new (ins_node_, 1);
@@ -506,6 +558,31 @@ mix_parser_add_raw (mix_parser_t *parser, mix_word_t word, guint lineno)
     }
 }
 
+void
+mix_parser_add_ins (mix_parser_t *parser, const mix_ins_t *new_ins,
+		    guint lineno)
+{
+  g_assert (parser != NULL && new_ins != NULL);
+  add_raw_ (parser, mix_ins_to_word_uncheck (*new_ins), lineno);
+}
+
+void
+mix_parser_add_raw (mix_parser_t *parser, mix_word_t word, guint lineno,
+                    gboolean is_con)
+{
+  g_assert (parser != NULL);
+  add_raw_ (parser, word, lineno);
+  if ( parser->status == MIX_PERR_NOCOMP || parser->status == MIX_PERR_OK )
+    {
+      if (is_con)
+        parser->con_list = g_slist_append (parser->con_list,
+                                           GUINT_TO_POINTER (lineno));
+      else
+        parser->alf_list = g_slist_append (parser->alf_list,
+                                           GUINT_TO_POINTER (lineno));
+    }
+}
+
 /* Error handling */
 void
 mix_parser_log_error (mix_parser_t *parser, mix_parser_err_t error,
