@@ -1,24 +1,24 @@
 /* -*-c-*- -------------- mixvm_loop.c :
  * Implementation of mix vm command loop.
  * ------------------------------------------------------------------
- *  $Id: mixvm_loop.c,v 1.10 2002/04/08 01:59:38 jao Exp $
+ *  $Id: mixvm_loop.c,v 1.14 2004/06/12 01:08:48 jao Exp $
  * ------------------------------------------------------------------
- * Copyright (C) 2000, 2001, 2002 Free Software Foundation, Inc.
- *  
+ * Copyright (C) 2000, 2001, 2002, 2004 Free Software Foundation, Inc.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *  
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *  
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *  
+ *
  */
 
 
@@ -60,7 +60,8 @@ readline (char *prompt)
 /* A static variable for holding the line. */
 static char *line_read = (char *)NULL;
 
-static char *PROMPT = NULL;
+#define PROMPT_LEN 128
+static char PROMPT[PROMPT_LEN + 1] = {'M', 'I', 'X', ' ', '>'};
 static const char *CONFIG_FILE_ = "mixvm.config";
 static const char *PROMPT_KEY_ = "Prompt";
 
@@ -75,14 +76,14 @@ rl_gets ()
       g_free (line_read);
       line_read = (char *)NULL;
     }
-     
+
   /* Get a line from the user. */
   line_read = readline ((char *)PROMPT);
-     
+
   /* If the line has any text in it, save it on the history. */
   if (line_read && *line_read)
     add_history (line_read);
-     
+
   return (line_read);
 }
 
@@ -95,7 +96,6 @@ init_mixvm_ (const gchar *file, gboolean use_emacs)
 {
   static const gchar *HISTORY_FILE = "mixvm.history";
   static gint HISTORY_SIZE = 100;
-  const char *prompt = NULL;
   config_ = mix_config_new (NULL, CONFIG_FILE_);
 
   mix_config_set_autosave (config_, TRUE);
@@ -103,31 +103,26 @@ init_mixvm_ (const gchar *file, gboolean use_emacs)
     mix_config_set_history_file (config_, HISTORY_FILE);
   if (mix_config_get_history_size (config_) == 0)
     mix_config_set_history_size (config_, HISTORY_SIZE);
-  
-  prompt = mix_config_get (config_, PROMPT_KEY_);
-  if (!prompt) prompt = "MIX > ";
-  mix_vmloop_set_prompt (prompt);
-  
+
+  mix_vmloop_set_prompt (mix_config_get (config_, PROMPT_KEY_));
+
   return mixvm_cmd_init (config_, (char *)file, use_emacs);
 }
 
 void
-mix_vmloop_set_prompt (const gchar *prompt) 
+mix_vmloop_set_prompt (const gchar *prompt)
 {
   if (prompt)
     {
-      if (PROMPT) g_free (PROMPT);
-      if (prompt[0] == '"') prompt++;
-      PROMPT = g_strdup (prompt);
-      if (PROMPT[strlen (PROMPT) - 1] == '"') PROMPT[strlen (PROMPT) - 1] = 0;
-      mix_config_update (config_, PROMPT_KEY_, PROMPT);
+      g_snprintf (PROMPT, PROMPT_LEN, "%s ", prompt);
+      mix_config_update (config_, PROMPT_KEY_, prompt);
     }
 }
 
 static void
 loop_ (int argc, char *argv[])
 {
-  while ( mixvm_cmd_exec (rl_gets ()) ) 
+  while ( mixvm_cmd_exec (rl_gets ()) )
     ;
   mix_config_delete (config_);
 }
@@ -147,16 +142,17 @@ mix_vmloop (int argc, char *argv[], gboolean initfile,
 
 /* run a program and exit */
 void
-mix_vmrun (const gchar *code_file, gboolean dump, gboolean ptime) 
+mix_vmrun (const gchar *code_file, gboolean dump, gboolean ptime)
 {
-  gchar *time_cmd = ptime? g_strdup("stime on") : g_strdup("stime off");
-  gchar *run_cmd = g_strdup("run");
-  gchar *dump_cmd = dump? g_strdup("pall") : NULL;
+  gchar *time_cmd = ptime? g_strdup ("stime on") : g_strdup ("stime off");
+  gchar *run_cmd = g_strdup ("run");
+  gchar *dump_cmd = dump? g_strdup ("pall") : NULL;
   gboolean result;
-  
+
   init_mixvm_ (code_file, FALSE);
   result = mixvm_cmd_exec (time_cmd) && mixvm_cmd_exec (run_cmd);
   if (result && dump) mixvm_cmd_exec (dump_cmd);
+  mix_config_set_autosave (config_, FALSE);
   mix_config_delete (config_);
   g_free(time_cmd);
   g_free(run_cmd);
